/* **********************************************************
 * Copyright 1998 VMware, Inc.  All rights reserved. -- VMware Confidential
 * **********************************************************/

/*
 *  hostif.h - Platform dependent interface for supporting 
 *             the vmx86 device driver. 
 */


#ifndef _HOSTIF_H_
#define _HOSTIF_H_

#define INCLUDE_ALLOW_MODULE
#include "includeCheck.h"

#include "vmx86.h"

/*
 * Host-specific definitions. 
 */
#if !defined(linux) && !defined(WINNT_DDK)
#error "ONLY Linux or NT defined for now "
#endif


#ifdef linux
#   define IRQ_HOST_INTR1_BASE 0x20
#   define IRQ_HOST_INTR2_BASE 0x28

#   ifdef KERNEL_2_1
/* See arch/i386/kernel/irq.h in the kernel source --hpreg */
#      define IRQ0_TRAP_VECTOR 0x51
#   endif
#endif

#ifdef WINNT_DDK
#   define IRQ_HOST_INTR1_BASE 0x30
#   define IRQ_HOST_INTR2_BASE 0x38
#endif

EXTERN void  HostIF_InitSpinLock(void);
EXTERN int   HostIF_Init(VMDriver *vm);
EXTERN MPN   HostIF_LookupUserMPN(VMDriver *vm, void *addr);
EXTERN void *HostIF_UserToDriverPtr(VMDriver *vm, void *addr);
EXTERN void  HostIF_InitFP(VMDriver *vm);
EXTERN void  HostIF_InitEvent(VMDriver *vm);

EXTERN void *HostIF_AllocKernelMem(size_t size, int nonPageable);
EXTERN void  HostIF_FreeKernelMem(void *ptr);

EXTERN void *HostIF_AllocPage(void);
EXTERN void  HostIF_FreePage(void *ptr);

/*
 * XXX Those are currently only needed on Windows, but they should go away, as
 *     those functions are _not_ part of the platform-specific interface that
 *     the cross-platform code uses --hpreg
 */
EXTERN void *HostIF_AllocLowPages(VMDriver *vm, size_t numPages);
EXTERN int   HostIF_FreeLowPages(VMDriver *vm, void *ptr);

EXTERN int   HostIF_CopyFromUser(void *dst, void *src, int len);
EXTERN int   HostIF_CopyToUser(void *dst, void *src, int len);
EXTERN int   HostIF_MarkPageDirty(void *addr);
EXTERN MPN   HostIF_LockPage(VMDriver *vm, void *addr);
EXTERN int   HostIF_UnlockPage(VMDriver *vm, void *addr);
EXTERN VPN   HostIF_GetRecycledPage(VMDriver *vm);
EXTERN int   HostIF_ReleaseAnonPage(VMDriver *vm, MPN mpn);
EXTERN int   HostIF_FreeAllResources(VMDriver *vm);
EXTERN VA    HostIF_CheckMemory(VMDriver *vm);
EXTERN int   HostIF_IsMPNLocked(VMDriver *vm, MPN mpn);
EXTERN uint64 HostIF_ReadUptime(void);
EXTERN uint64 HostIF_UptimeFrequency(void);

EXTERN void  HostIF_GlobalVMLock(int callerID);
EXTERN void  HostIF_GlobalVMUnLock(int callerID);

EXTERN MA    HostIF_APICBase(VMDriver *vm, Bool setVMPtr, Bool probe);
EXTERN MA    HostIF_IOAPICBase(VMDriver *vm);

EXTERN void  HostIF_LockKernel(void);
EXTERN void  HostIF_UnLockKernel(void);
EXTERN void  HostIF_GetCPUID(int index, uint32* regs);
EXTERN uint64 HostIF_RDMSR(int index);
EXTERN void  HostIF_WRMSR(int index, uint64 qword);

EXTERN int   HostIF_SemaphoreWait(VMDriver *vm, Vcpuid vcpuid, int fd);
EXTERN int   HostIF_SemaphoreSignal(int fd);
EXTERN void  HostIF_SemaphoreForceWakeup(VMDriver *vm, Vcpuid vcpuid);
EXTERN int   HostIF_IPI(HostThread thread);

#ifdef WIN32
EXTERN void  HostIF_SetUserCallHandle(VMDriver *vm, int h);
EXTERN int   HostIF_SyncReadTSCs(uint64* TSCs, uint32 cpuSet);
EXTERN int   HostIF_SyncWriteTSCs(const uint32* TSCs, uint32 cpuSet);
EXTERN void  HostIF_SynchronizeTSCs(void);
#endif
EXTERN void  HostIF_UserCall(VMDriver *vm, Vcpuid vcpuid);
EXTERN Bool  HostIF_UserCallWait(VMDriver *vm, Vcpuid vcpuid);
EXTERN void  HostIF_AwakenVcpu(VMDriver *vm, Vcpuid vcpuid);
EXTERN void  HostIF_AckUserCall(VMDriver *vm, Vcpuid vcpuid);

#ifdef WIN32
#define HostIF_Reschedule()
#else
#include "compat_sched.h"
#define HostIF_Reschedule() cond_resched()
#endif

#if defined(USE_PERFCTRS_HOSTED) && defined(linux)
EXTERN MA HostIF_APICEnable(VMDriver *vm);
EXTERN Bool HostIF_APICSetPerfCtr(VMDriver *vm, int gate);
EXTERN void HostIF_APICAck(VMDriver *vm);
#endif 

#endif // ifdef _HOSTIF_H_
